"""Graph Insights command group"""

import time

import click
from click import Argument, Context
from click.shell_completion import CompletionItem
from cmem.cmempy.api import get_json, request
from cmem.cmempy.config import get_dp_api_endpoint
from requests import HTTPError

from cmem_cmemc.command import CmemcCommand
from cmem_cmemc.command_group import CmemcGroup
from cmem_cmemc.completion import NOT_SORTED, finalize_completion, graph_uris
from cmem_cmemc.context import ApplicationContext
from cmem_cmemc.exceptions import CmemcError
from cmem_cmemc.object_list import (
    DirectListPropertyFilter,
    DirectValuePropertyFilter,
    ObjectList,
    transform_lower,
)
from cmem_cmemc.string_processor import GraphLink, TimeAgo
from cmem_cmemc.utils import get_graphs_as_dict, struct_to_table


def get_api_url(path: str = "") -> str:
    """Get URLs of the graph insights API.

    Constructs the full URL for accessing graph insights API endpoints by combining
    the DataPlatform API endpoint with the semspect extension base path and an
    optional resource path.

    Args:
        path: The API resource path to append to the base URL. Defaults to an empty
            string for the root endpoint.

    Returns:
        The complete URL for the specified graph insights API endpoint.

    Example:
        >>> get_api_url()
        'https://example.com/dataplatform/api/ext/semspect'
        >>> get_api_url("/snapshot/status")
        'https://example.com/dataplatform/api/ext/semspect/snapshot/status'

    """
    base_url = get_dp_api_endpoint() + "/api/ext/semspect"
    return f"{base_url}{path}"


def is_available() -> bool:
    """Check availability of graph insights endpoints

    {
     "isActive": true,
     "isUserAllowed": true
    }
    """
    try:
        data: dict[str, bool] = get_json(get_api_url())
    except HTTPError:
        return False
    return bool(data["isActive"] is True and data["isUserAllowed"] is True)


def check_availability(ctx: click.Context) -> None:
    """Check availability of graph insights endpoints or raise an exception"""
    if is_available():
        return
    app: ApplicationContext = ctx.obj
    raise CmemcError(app, "Graph Insights is not available.")


def get_snapshots(ctx: click.Context) -> list[dict[str, str | bool | list[str]]]:
    """Get the snapshot list (all snapshots)"""
    check_availability(ctx)
    data: list[dict[str, str | bool | list[str]]] = get_json(
        get_api_url("/snapshot/status"), params={"includeManagementOnly": True}
    )
    return data


def complete_snapshot_ids(ctx: Context, param: Argument, incomplete: str) -> list[CompletionItem]:  # noqa: ARG001
    """Provide auto-completion for snapshot Ids"""
    ApplicationContext.set_connection_from_params(ctx.find_root().params)
    if not is_available():
        return []
    snapshots = get_snapshots(ctx)
    snapshots = sorted(
        snapshots, key=lambda snapshot: snapshot["updateInfoTimestamp"], reverse=True
    )
    options = [
        (
            snapshot["databaseId"],
            f"{snapshot['mainGraphSynced']} ({snapshot['updateInfoTimestamp']})",
        )
        for snapshot in snapshots
    ]
    return finalize_completion(candidates=options, incomplete=incomplete, sort_by=NOT_SORTED)


snapshot_list = ObjectList(
    name="insight snapshots",
    get_objects=get_snapshots,
    filters=[
        DirectValuePropertyFilter(
            name="id",
            description="Snapshots with a specific id.",
            property_key="databaseId",
            transform=transform_lower,
        ),
        DirectValuePropertyFilter(
            name="main-graph",
            description="Snapshots with a specific main graph.",
            property_key="mainGraphSynced",
        ),
        DirectValuePropertyFilter(
            name="status",
            description="Snapshots with a specific status.",
            property_key="status",
        ),
        DirectListPropertyFilter(
            name="affected-graph",
            description="Snapshots with a specific affected graph (main or sub-graphs).",
            property_key="allGraphsSynced",
        ),
        DirectValuePropertyFilter(
            name="valid",
            description="Snapshots with a specific validity indicator.",
            property_key="isValid",
            transform=transform_lower,
        ),
    ],
)


@click.command(cls=CmemcCommand, name="list")
@click.option(
    "--filter",
    "filter_",
    type=(str, str),
    help=snapshot_list.get_filter_help_text(),
    shell_complete=snapshot_list.complete_values,
    multiple=True,
)
@click.option("--raw", is_flag=True, help="Outputs raw JSON response.")
@click.option(
    "--id-only",
    is_flag=True,
    help="Return the snapshot IDs only. This is useful for piping the IDs into other commands.",
)
@click.pass_context
def list_command(ctx: Context, filter_: tuple[tuple[str, str]], id_only: bool, raw: bool) -> None:
    """List graph insight snapshots.

    Graph Insights Snapshots are identified by an ID.
    """
    check_availability(ctx)
    app: ApplicationContext = ctx.obj
    snapshots = snapshot_list.apply_filters(ctx=ctx, filter_=filter_)

    if id_only:
        for _ in snapshots:
            click.echo(_["databaseId"])
        return

    if raw:
        app.echo_info_json(snapshots)
        return

    graphs = get_graphs_as_dict()
    table = []
    for _ in snapshots:
        id_ = _["databaseId"]
        main_graph = _["mainGraphSynced"]
        updated = _["updateInfoTimestamp"]
        status = _["status"]
        is_valid = _["isValid"]
        if main_graph not in graphs:
            main_graph = rf"\[missing: {main_graph}]"
        table.append([id_, main_graph, updated, status, is_valid])

    app.echo_info_table(
        table,
        headers=["ID", "Main Graph", "Updated", "Status", "Valid"],
        sort_column=0,
        cell_processing={1: GraphLink(), 2: TimeAgo()},
        empty_table_message="No graph insight snapshots found.",
    )


@click.command(cls=CmemcCommand, name="delete")
@click.argument("SNAPSHOT_ID", type=str, shell_complete=complete_snapshot_ids, required=False)
@click.option(
    "--filter",
    "filter_",
    type=(str, str),
    help=snapshot_list.get_filter_help_text(),
    shell_complete=snapshot_list.complete_values,
    multiple=True,
)
@click.option("-a", "--all", "all_", is_flag=True, help="Delete all snapshots.")
@click.pass_context
def delete_command(
    ctx: Context, snapshot_id: str | None, filter_: tuple[tuple[str, str]], all_: bool
) -> None:
    """Delete a graph insight snapshot.

    Graph Insight Snapshots are identified by an ID.
    To get a list of existing snapshots,
    execute the `graph insights list` command or use tab-completion.
    """
    check_availability(ctx)
    app: ApplicationContext = ctx.obj
    if snapshot_id is None and not filter_ and not all_:
        raise click.UsageError("Either provide a snapshot ID or a filter, or use the --all flag.")

    if all_:
        app.echo_info("Deleting all snapshots ... ", nl=False)
        request(method="DELETE", uri=get_api_url("/snapshot"))
        app.echo_success("done")
        return

    filter_to_apply = list(filter_) if filter_ else []
    if snapshot_id:
        filter_to_apply.append(("id", snapshot_id))
    snapshots_to_delete = snapshot_list.apply_filters(ctx=ctx, filter_=filter_to_apply)

    if not snapshots_to_delete and snapshot_id:
        raise click.ClickException(f"Snapshot ID '{snapshot_id}' does not exist.")

    if not snapshots_to_delete and not snapshot_id:
        raise click.ClickException("No snapshots found to delete.")

    for _ in snapshots_to_delete:
        id_to_delete = _["databaseId"]
        app.echo_info(f"Deleting snapshot {id_to_delete} ... ", nl=False)
        request(method="DELETE", uri=get_api_url(f"/snapshot/{id_to_delete}"))
        app.echo_success("done")


def wait_for_snapshot(snapshot_id: str, polling_interval: int) -> None:
    """Poll until the snapshot reaches 'DONE' status."""
    while True:
        snapshot: dict[str, str | bool | list[str]] = get_json(
            get_api_url(f"/snapshot/status/{snapshot_id}")
        )
        if snapshot.get("status") == "DONE":
            break
        time.sleep(polling_interval)


@click.command(cls=CmemcCommand, name="create")
@click.argument("iri", type=click.STRING, shell_complete=graph_uris)
@click.option("--wait", is_flag=True, help="Wait until snapshot creation is done.")
@click.option(
    "--polling-interval",
    type=click.IntRange(min=0, max=60),
    show_default=True,
    default=1,
    help="How many seconds to wait between status polls. Status polls are"
    " cheap, so a higher polling interval is most likely not needed.",
)
@click.pass_context
def create_command(ctx: Context, iri: str, wait: bool, polling_interval: int) -> None:
    """Create or update a graph insight snapshot.

    Create a graph insight snapshot for a given graph.
    If the snapshot already exists, it is hot-swapped after re-creation.
    The snapshot contains only the (imported) graphs the requesting user can read.
    """
    check_availability(ctx)
    app: ApplicationContext = ctx.obj
    app.echo_info(f"Create / Update graph snapshot for graph {iri} ... ", nl=False)
    snapshot_id = request(
        method="POST", uri=get_api_url("/snapshot"), params={"contextGraph": iri}
    ).text
    app.echo_success("started", nl=not wait)
    if wait:
        app.echo_info(" ... ", nl=False)
        wait_for_snapshot(snapshot_id, polling_interval)
        app.echo_success("created")


@click.command(cls=CmemcCommand, name="update")
@click.argument("SNAPSHOT_ID", type=str, shell_complete=complete_snapshot_ids, required=False)
@click.option(
    "--filter",
    "filter_",
    type=(str, str),
    help=snapshot_list.get_filter_help_text(),
    shell_complete=snapshot_list.complete_values,
    multiple=True,
)
@click.option("-a", "--all", "all_", is_flag=True, help="Delete all snapshots.")
@click.option("--wait", is_flag=True, help="Wait until snapshot creation is done.")
@click.option(
    "--polling-interval",
    type=click.IntRange(min=0, max=60),
    show_default=True,
    default=1,
    help="How many seconds to wait between status polls. Status polls are"
    " cheap, so a higher polling interval is most likely not needed.",
)
@click.pass_context
def update_command(  # noqa: PLR0913
    ctx: Context,
    snapshot_id: str | None,
    filter_: tuple[tuple[str, str]],
    all_: bool,
    wait: bool,
    polling_interval: int,
) -> None:
    """Update a graph insight snapshot.

    Update a graph insight snapshot.
    After the update, the snapshot is hot-swapped.
    """
    check_availability(ctx)
    app: ApplicationContext = ctx.obj
    if snapshot_id is None and not filter_ and not all_:
        raise click.UsageError("Either provide a snapshot ID or a filter, or use the --all flag.")

    filter_to_apply = list(filter_) if filter_ else []
    if snapshot_id:
        filter_to_apply.append(("id", snapshot_id))
    snapshots_to_update = snapshot_list.apply_filters(ctx=ctx, filter_=filter_to_apply)

    if all_:
        snapshots_to_update = get_snapshots(ctx)

    if not snapshots_to_update and snapshot_id:
        raise click.ClickException(f"Snapshot ID '{snapshot_id}' does not exist.")

    if not snapshots_to_update and not snapshot_id:
        raise click.ClickException("No snapshots found to update.")

    for _ in snapshots_to_update:
        id_to_update = _["databaseId"]
        app.echo_info(f"Update snapshot {id_to_update} ... ", nl=False)
        request(method="PUT", uri=get_api_url(f"/snapshot/{id_to_update}"))
        app.echo_success("started", nl=not wait)
        if wait:
            app.echo_info(" ... ", nl=False)
            wait_for_snapshot(id_to_update, polling_interval)
            app.echo_success("updated")


@click.command(cls=CmemcCommand, name="inspect")
@click.argument("SNAPSHOT_ID", type=str, shell_complete=complete_snapshot_ids)
@click.option("--raw", is_flag=True, help="Outputs raw JSON.")
@click.pass_context
def inspect_command(ctx: Context, snapshot_id: str, raw: bool) -> None:
    """Inspect the metadata of a graph insight snapshot."""
    check_availability(ctx)
    app: ApplicationContext = ctx.obj
    snapshot: dict[str, str | bool | list[str]] = get_json(
        get_api_url(f"/snapshot/status/{snapshot_id}")
    )
    if raw:
        app.echo_info_json(snapshot)
    else:
        table = struct_to_table(snapshot)
        app.echo_info_table(table, headers=["Key", "Value"], sort_column=0)


@click.group(cls=CmemcGroup, name="insights")
def insights_group() -> CmemcGroup:  # type: ignore[empty-body]
    """List, create, delete and inspect graph insight snapshots.

    Graph Insight Snapshots are identified by an ID.
    To get a list of existing snapshots,
    execute the `graph insights list` command or use tab-completion.
    """


insights_group.add_command(list_command)
insights_group.add_command(delete_command)
insights_group.add_command(create_command)
insights_group.add_command(update_command)
insights_group.add_command(inspect_command)



"""Build project file commands for cmemc."""

import re

import click
from click import ClickException, Context, UsageError
from cmem.cmempy.config import get_cmem_base_uri
from cmem.cmempy.workspace.projects.resources import get_all_resources
from cmem.cmempy.workspace.projects.resources.resource import (
    create_resource,
    delete_resource,
    get_resource_metadata,
    get_resource_response,
    get_resource_usage_data,
    resource_exist,
)

from cmem_cmemc import completion
from cmem_cmemc.command import CmemcCommand
from cmem_cmemc.command_group import CmemcGroup
from cmem_cmemc.context import ApplicationContext
from cmem_cmemc.exceptions import CmemcError
from cmem_cmemc.parameter_types.path import ClickSmartPath
from cmem_cmemc.smart_path import SmartPath as Path
from cmem_cmemc.string_processor import FileSize, TimeAgo
from cmem_cmemc.utils import check_or_select_project, split_task_id, struct_to_table

RESOURCE_FILTER_TYPES = ["project", "regex"]
RESOURCE_FILTER_TYPES_HIDDEN = ["ids"]
RESOURCE_FILTER_TEXT = (
    "Filter file resources based on metadata. "
    f"First parameter CHOICE can be one of {RESOURCE_FILTER_TYPES!s}"
    ". The second parameter is based on CHOICE, e.g. a project "
    "ID or a regular expression string."
)


def _upload_file_resource(
    app: ApplicationContext,
    project_id: str,
    local_file_name: str,
    remote_file_name: str,
    replace: bool,
) -> None:
    """Upload a local file as a dataset resource to a project.

    Args:
    ----
        app: the click cli app context.
        project_id: The project ID in the workspace.
        local_file_name: The path to the local file name
        remote_file_name: The remote file name
        replace: Replace resource if needed.

    Raises:
    ------
        ValueError: if resource exists and no replace

    """
    exist = resource_exist(project_name=project_id, resource_name=remote_file_name)
    if exist and not replace:
        raise CmemcError(
            app,
            f"A file resource with the name '{remote_file_name}' already "
            "exists in this project. \n"
            "Please rename the file or use the '--replace' "
            "parameter in order to overwrite the remote file.",
        )
    if exist:
        app.echo_info(
            f"Replace content of {remote_file_name} with content from "
            f"{local_file_name} in project {project_id} ... ",
            nl=False,
        )
    else:
        app.echo_info(
            f"Upload {local_file_name} as a file resource "
            f"{remote_file_name} to project {project_id} ... ",
            nl=False,
        )
    create_resource(
        project_name=project_id,
        resource_name=remote_file_name,
        file_resource=ClickSmartPath.open(local_file_name),
        replace=replace,
    )
    app.echo_success("done")


def _get_resources_filtered(
    resources: list[dict], filter_name: str, filter_value: str | tuple[str, ...]
) -> list[dict]:
    """Get file resources but filtered according to name and value."""
    # check for correct filter names (filter ids is used internally only)
    if filter_name not in RESOURCE_FILTER_TYPES + RESOURCE_FILTER_TYPES_HIDDEN:
        raise UsageError(
            f"{filter_name} is an unknown filter name. " f"Use one of {RESOURCE_FILTER_TYPES}."
        )
    # filter by ID list
    if filter_name == "ids":
        return [_ for _ in resources if _["id"] in filter_value]
    # filter by project
    if filter_name == "project":
        return [_ for _ in resources if _["project"] == str(filter_value)]
    # filter by regex
    if filter_name == "regex":
        return [_ for _ in resources if re.search(str(filter_value), _["name"])]
    # return unfiltered list
    return resources


@click.command(cls=CmemcCommand, name="list")
@click.option("--raw", is_flag=True, help="Outputs raw JSON.")
@click.option(
    "--id-only",
    is_flag=True,
    help="Lists only resource IDs and no other metadata. "
    "This is useful for piping the IDs into other commands.",
)
@click.option(
    "--filter",
    "filters_",
    multiple=True,
    type=(str, str),
    shell_complete=completion.resource_list_filter,
    help=RESOURCE_FILTER_TEXT,
)
@click.pass_obj
def list_command(
    app: ApplicationContext, raw: bool, id_only: bool, filters_: tuple[tuple[str, str], ...]
) -> None:
    """List available file resources.

    Outputs a table or a list of file resources.
    """
    resources = get_all_resources()
    for _ in filters_:
        filter_name, filter_value = _
        resources = _get_resources_filtered(resources, filter_name, filter_value)
    if raw:
        app.echo_info_json(resources)
        return
    if id_only:
        for _ in sorted(_["id"] for _ in resources):
            app.echo_result(_)
        return
    # output a user table
    table = []
    headers = ["ID", "Modified", "Size"]
    for _ in resources:
        row = [
            _["id"],
            _["modified"],
            _["size"],
        ]
        table.append(row)

    caption = f"{len(table)} files of {get_cmem_base_uri()}"
    empty_note = "No resources found."
    if len(filters_) > 0:
        caption += " (filtered)"
        empty_note = "No resources found for these filters."

    app.echo_info_table(
        table,
        headers=headers,
        sort_column=0,
        cell_processing={1: TimeAgo(), 2: FileSize()},
        caption=caption,
        empty_table_message=f"{empty_note} "
        "Use the `dataset create` command to create a new file-based dataset, or "
        "the `project file upload` command to create only a file resource.",
    )


@click.command(cls=CmemcCommand, name="delete")
@click.argument("resource_ids", nargs=-1, type=click.STRING, shell_complete=completion.resource_ids)
@click.option("--force", is_flag=True, help="Delete resource even if in use by a task.")
@click.option(
    "-a",
    "--all",
    "all_",
    is_flag=True,
    help="Delete all resources. " "This is a dangerous option, so use it with care.",
)
@click.option(
    "--filter",
    "filters_",
    multiple=True,
    type=(str, str),
    shell_complete=completion.resource_list_filter,
    help=RESOURCE_FILTER_TEXT,
)
@click.pass_obj
def delete_command(
    app: ApplicationContext,
    resource_ids: tuple[str, ...],
    force: bool,
    all_: bool,
    filters_: tuple[tuple[str, str], ...],
) -> None:
    """Delete file resources.

    There are three selection mechanisms: with specific IDs - only those
    specified resources will be deleted; by using --filter - resources based
    on the filter type and value will be deleted; by using --all, which will
    delete all resources.
    """
    if resource_ids == () and not all_ and filters_ == ():
        raise UsageError(
            "Either specify at least one resource ID or use the --all or "
            "--filter options to specify resources for deletion."
        )

    resources = get_all_resources()
    if len(resource_ids) > 0:
        for resource_id in resource_ids:
            if resource_id not in [_["id"] for _ in resources]:
                raise ClickException(f"Resource {resource_id} not available.")
        # "filter" by id
        resources = _get_resources_filtered(resources, "ids", resource_ids)
    for _ in filters_:
        resources = _get_resources_filtered(resources, _[0], _[1])

    # avoid double removal as well as sort IDs
    processed_ids = sorted({_["id"] for _ in resources}, key=lambda v: v.lower())
    count = len(processed_ids)
    for current, resource_id in enumerate(processed_ids, start=1):
        current_string = str(current).zfill(len(str(count)))
        app.echo_info(f"Delete resource {current_string}/{count}: {resource_id} ... ", nl=False)
        project_id, resource_local_id = split_task_id(resource_id)
        usage = get_resource_usage_data(project_id, resource_local_id)
        if len(usage) > 0:
            app.echo_error(f"in use by {len(usage)} task(s)", nl=False)
            if force:
                app.echo_info(" ... ", nl=False)
            else:
                app.echo_info("")
                continue
        delete_resource(project_name=project_id, resource_name=resource_local_id)
        app.echo_success("deleted")


@click.command(cls=CmemcCommand, name="download")
@click.argument("resource_ids", nargs=-1, type=click.STRING, shell_complete=completion.resource_ids)
@click.option(
    "--output-dir",
    default=".",
    show_default=True,
    type=ClickSmartPath(writable=True, file_okay=False),
    help="The directory where the downloaded files will be saved. "
    "If this directory does not exist, it will be created.",
)
@click.option(
    "--replace",
    is_flag=True,
    help="Replace existing files. This is a dangerous option, " "so use it with care!",
)
@click.pass_obj
def download_command(
    app: ApplicationContext, resource_ids: tuple[str], output_dir: str, replace: bool
) -> None:
    """Download file resources to the local file system.

    This command downloads one or more file resources from projects to your local
    file system. Files are saved with their resource names in the output directory.

    Resources are identified by their IDs in the format PROJECT_ID:RESOURCE_NAME.

    Example: cmemc project file download my-proj:my-file.csv

    Example: cmemc project file download my-proj:file1.csv my-proj:file2.csv --output-dir /tmp
    """
    import os

    if not resource_ids:
        raise UsageError(
            "At least one resource ID must be specified. "
            "Use 'project file list' to see available resources."
        )

    count = len(resource_ids)
    for current, resource_id in enumerate(resource_ids, start=1):
        try:
            project_id, resource_name = split_task_id(resource_id)
        except ValueError:
            app.echo_error(f"Invalid resource ID format: {resource_id}")
            continue

        # Build output path
        output_path = os.path.normpath(str(Path(output_dir) / resource_name))

        app.echo_info(
            f"Download resource {current}/{count}: {resource_id} to {output_path} ... ",
            nl=False,
        )

        if Path(output_path).exists() and replace is not True:
            app.echo_error("target file exists")
            continue

        # Create parent directory if it doesn't exist
        Path(output_path).parent.mkdir(exist_ok=True, parents=True)

        try:
            with (
                get_resource_response(project_id, resource_name) as response,
                click.open_file(output_path, "wb") as resource_file,
            ):
                for chunk in response.iter_content(chunk_size=8192):
                    resource_file.write(chunk)
            app.echo_success("done")
        except (OSError, ClickException) as error:
            app.echo_error(f"failed: {error!s}")
            continue


@click.command(cls=CmemcCommand, name="upload")
@click.argument(
    "input_path",
    required=True,
    type=ClickSmartPath(
        allow_dash=False, dir_okay=False, readable=True, exists=True, remote_okay=True
    ),
)
@click.option(
    "--project",
    "project_id",
    type=click.STRING,
    shell_complete=completion.project_ids,
    help="The project where you want to upload the file. If there is "
    "only one project in the workspace, this option can be omitted.",
)
@click.option(
    "--path",
    "remote_name",
    type=click.STRING,
    shell_complete=completion.resource_paths,
    help="The path/name of the file resource in the project (e.g., 'data/file.csv'). "
    "If not specified, the local file name will be used.",
)
@click.option(
    "--replace",
    is_flag=True,
    help="Replace existing file resource. This is a dangerous option, " "so use it with care!",
)
@click.pass_obj
def upload_command(
    app: ApplicationContext, input_path: str, project_id: str, remote_name: str, replace: bool
) -> None:
    """Upload a file to a project.

    This command uploads a file to a project as a file resource.

    Note: If you want to create a dataset from your file, the `dataset create`
    command is maybe the better option.

    Example: cmemc project file upload my-file.csv --project my-project
    """
    project_id = check_or_select_project(app, project_id)
    local_file_name = Path(input_path).name

    if remote_name and remote_name.endswith("/"):
        app.echo_warning(
            f"Remote path ends with a slash, so the local file name is appended: {local_file_name}."
        )
        remote_name = remote_name + local_file_name

    # Use local filename if remote name not specified
    if not remote_name:
        remote_name = local_file_name

    _upload_file_resource(
        app=app,
        remote_file_name=remote_name,
        project_id=project_id,
        local_file_name=input_path,
        replace=replace,
    )


@click.command(cls=CmemcCommand, name="inspect")
@click.argument("resource_id", type=click.STRING, shell_complete=completion.resource_ids)
@click.option("--raw", is_flag=True, help="Outputs raw JSON.")
@click.pass_obj
def inspect_command(app: ApplicationContext, resource_id: str, raw: bool) -> None:
    """Display all metadata of a file resource."""
    project_id, resource_id = split_task_id(resource_id)
    resource_data = get_resource_metadata(project_id, resource_id)
    if raw:
        app.echo_info_json(resource_data)
    else:
        table = struct_to_table(resource_data)
        app.echo_info_table(table, headers=["Key", "Value"], sort_column=0)


@click.command(cls=CmemcCommand, name="usage")
@click.argument("resource_id", type=click.STRING, shell_complete=completion.resource_ids)
@click.option("--raw", is_flag=True, help="Outputs raw JSON.")
@click.pass_obj
def usage_command(app: ApplicationContext, resource_id: str, raw: bool) -> None:
    """Display all usage data of a file resource."""
    project_id, resource_id = split_task_id(resource_id)
    usage = get_resource_usage_data(project_id, resource_id)
    if raw:
        app.echo_info_json(usage)
        return
    # output a user table
    table = []
    headers = ["Task ID", "Type", "Label"]
    for _ in usage:
        row = [project_id + ":" + _["id"], _["taskType"], _["label"]]
        table.append(row)
    app.echo_info_table(
        table,
        empty_table_message=f"The file resource '{resource_id}' is not used in "
        f"any task in project '{project_id}'.",
        headers=headers,
        sort_column=2,
    )


@click.group(
    cls=CmemcGroup,
    hidden=True,
)
@click.pass_context
def resource(ctx: Context) -> None:
    """List, inspect or delete dataset file resources.

    File resources are identified by their paths and project IDs.

    Warning: This command group is deprecated and will be removed with the next major release.
    Please use the `project file` command group instead.
    """
    app: ApplicationContext = ctx.obj
    app.echo_warning(
        "The 'dataset resource' command group is deprecated and will be removed with the next"
        " major release. Please use the 'project file' command group instead.",
    )


@click.group(cls=CmemcGroup)
def file() -> CmemcGroup:  # type: ignore[empty-body]
    """List, inspect, up-/download or delete project file resources.

    File resources are identified with a RESOURCE_ID which is a concatenation
    of its project ID and its relative path, e.g. `my-project:path-to/table.csv`.

    Note: To get a list of existing file resources, execute the `project file list` command
    or use tab-completion.
    """


resource.add_command(list_command)
resource.add_command(delete_command)
resource.add_command(inspect_command)
resource.add_command(usage_command)

file.add_command(list_command)
file.add_command(delete_command)
file.add_command(download_command)
file.add_command(upload_command)
file.add_command(inspect_command)
file.add_command(usage_command)
